#!/usr/bin/env python

#  GnomeArtScreenlet by Whise aka Helder Fraga  
#
# INFO:
# - A Screenlet that gets wallpapers from art.gnome.org website
# 
# TODO:
# - More costumization in options giving the user more control
# - ...

import screenlets
from screenlets.options import FileOption, IntOption, FloatOption, StringOption, BoolOption
from screenlets import DefaultMenuItem
import cairo
import gtk
import pango
from urllib2 import urlopen
import gobject
try:
	import Image
except:
	print 'Error - Please install python image module'
import os
import commands
import random


class GnomeArtScreenlet (screenlets.Screenlet):
	"""A Screenlet that gets wallpapers from art.gnome.org website or from a folder you select and sets them as your wallpaper image . It also shows images in the screenlet frame .You can add new images by 		drag&amp;drop them into the screenlet's window area."""
	
	# --------------------------------------------------------------------------
	# meta-info, options
	# --------------------------------------------------------------------------
	
	__name__		= 'GnomeArtScreenlet'
	__version__		= '0.4'
	__author__		= 'Helder Fraga aka whise'
	__desc__		= __doc__
	
	# attributes
	__image = None
	__timeout = None
	# editable options
	update_interval = 60
	image_filename 	= ''
	image_scale		= 1.40
	image_offset_x	= 10
	image_offset_y	= 28
	slide = True
	folders = '' #commands.getoutput("echo $HOME")
	use_types = ['.jpg', '.gif', '.png']
	engine = ''
	engine1 = ''
	engine_sel = ['directory', 'gnome_art']	
	paint_menu = False
	saveit = False
	imagenumber = 0
	# --------------------------------------------------------------------------
	# constructor and internals
	# --------------------------------------------------------------------------
	
	def __init__ (self, **keyword_args):
		# call super (and enable drag/drop)
		screenlets.Screenlet.__init__(self, width=200, height=200,
			uses_theme=True, drag_drop=True, **keyword_args)
		# set theme
		self.theme_name = "default"
		# initially apply default image (for newly created instances)
		#self.image_filename = screenlets.PATH + '/Picframe/dali.png'
		# add default menuitems (all the standard ones)
		self.add_default_menuitems(DefaultMenuItem.XML)
		self.add_default_menuitems()
		
		# add option group to properties-dialog
		self.add_options_group('GnomeArt', 'GnomeArt-related settings ...')
		# add editable options
		#self.add_option(FileOption('GnomeArt', 'image_filename', 
		#	self.image_filename, 'Filename',
		#	'Filename of image to be shown in this GnomeArt ...')) 
		self.add_option(FloatOption('GnomeArt', 'image_scale', self.image_scale, 
			'Image Scale', 'Scale of image within this Picframe ...', 
			min=0.01, max=10.0, digits=2, increment=0.01))
		self.add_option(IntOption('GnomeArt', 'image_offset_x', 
			self.image_offset_x, 'Image Offset X', 'X-offset of upper left '+\
			'corner of the image within this Picframe ...', 
			min=0, max=self.width))
		self.add_option(IntOption('GnomeArt', 'image_offset_y', 
			self.image_offset_y, 'Image Offset Y', 'Y-offset of upper left '+\
			'corner of the image within this Picframe ...', 
			min=0, max=self.height))
		self.add_option(IntOption('GnomeArt', 'update_interval', 
			self.update_interval, 'Update interval', 
			'The interval for updating info (in seconds ,3660 = 1 day, 25620 = 1 week)', min=1, max=25620))
		self.add_option(StringOption('GnomeArt', 'engine', self.engine,'Select Engine', '',choices = self.engine_sel),realtime=False)
		self.add_option(StringOption('GnomeArt', 'folders', self.folders,'Select Folders', 'The folder where pictures are',))		
		self.add_option(BoolOption('GnomeArt', 'saveit',bool(self.saveit), 'Save downloaded wallpapers','Save downloaded wallpapers from gnomeart in this folder to?'))
		self.update_interval = self.update_interval

	def on_init(self):
		self.engine = self.engine
	def __setattr__ (self, name, value):

		screenlets.Screenlet.__setattr__(self, name, value)

		if name == 'engine':
			
				self.engine1 = value
				self.update()
		
			


		if name == "image_filename":
			#print "SET IMAGEFILENAME"
			# set self.__image to new image, if value is set and != current
			ret = True
			if value != '' and value != self.image_filename:	# ok?
				ret = self.set_image(value)
			# call super (if image has been successfully applied)
			if ret != False:
				screenlets.Screenlet.__setattr__(self, name, value)
			# update view
			self.redraw_canvas()
			#self.update_shape()



		if name == "update_interval":
			if value > 0:
				self.__dict__['update_interval'] = value
				if self.__timeout:
					gobject.source_remove(self.__timeout)
				self.__timeout = gobject.timeout_add(int(value * 1000), self.update)
			else:
				self.__dict__['update_interval'] = 1
				pass

	def set_image(self, filename):
		"""Set new image for this pictureframe (does NOT call redraw).
		TODO: support for more image-types (currently only png supported)"""
		# delete old image first
		print "Setting new image for GnomeArtScreenlet: %s" % filename
		if self.__image:
			self.__image.finish()
			del self.__image
		# and recreate image
		try:
			image = Image.open(filename)
			image = image.resize ((128,100), Image.ANTIALIAS)
			home = commands.getoutput("echo $HOME")
			image.save (home + '/wallpaper.png')
			img = cairo.ImageSurface.create_from_png(home + '/wallpaper.png')
			if img:
				self.__image = img
			
			#self.redraw_canvas()
			return True
		except Exception, ex:
			print 'Failed to load image '
		return False
	

	def fetch_image(self):
	 
	 if self.engine1 == 'gnome_art':		
		self.imagenumber = str(random.randrange(1860))
		zodiacfd = urlopen('http://art.gnome.org/backgrounds?page=' + self.imagenumber + '&sort_by=date&limit=1&view=list&resolution=all&order=DESC')
	
		#zodiacfd = urlopen('http://art.gnome.org/backgrounds/gnome/' + str(random.randrange(2400)) + '/')
		#print ('http://art.gnome.org/backgrounds/gnome/' + str(random.randrange(2400)))
		zodiachtml = zodiacfd.read()
		forecast = zodiachtml[zodiachtml.find("http://art.gnome.org"):]
		forecast = forecast[:forecast.find(chr(34) +'><img') ].strip()
		zodiacfd = urlopen(forecast)
		zodiachtml = zodiacfd.read()
		forecast = zodiachtml[zodiachtml.find("/download/"):]
		forecast = forecast[:forecast.find('class') -2].strip()
		
		zodiacfd = urlopen("http://art.gnome.org" + forecast)
		zodiachtml = zodiacfd.read()
		home = commands.getoutput("echo $HOME")
		
		fileObj = open( home + "/wallpaper.jpg","w") #// open for for write
		fileObj.write(zodiachtml)

		fileObj.close()
		
		self.image_filename =  home + "/wallpaper.jpg"
		os.system("gconftool-2 -t string -s /desktop/gnome/background/picture_filename " + self.image_filename)
		if  self.saveit == True  : 
			if self.folders != '': 
				os.system('cp ' + self.image_filename + ' ' + self.folders + '/' + self.imagenumber + '.jpg')
		forecast = self.image_filename

	 if self.engine1 == 'directory':
		imgs = []
		
		for d in self.folders: #for each item in folders
	  		if os.path.exists(self.folders) and os.path.isdir(self.folders): #is it a valid folder?
				for f in os.listdir(self.folders):                #get all files in that folder
					
		      			try:  #splitext[1] may fail
						if os.path.splitext(f)[1] in self.use_types: #is that file of proper type?
			                 		imgs.append(self.folders + os.sep + f)         #if so, add it to our list
					
			   		except:
			              		pass
	
		try:
			forecast = random.choice(imgs)  #get a random entry from our list
			os.system("gconftool-2 -t string -s /desktop/gnome/background/picture_filename " + chr(34) + forecast + chr(34))
			
		except:
			              		pass

	 try:return forecast	
	 except:
			              		pass
	# --------------------------------------------------------------------------
	# Screenlet handlers
	# --------------------------------------------------------------------------
	def update(self):
		#screenlets.show_error(self, 
		#		'Failed to load image "%s": %s (only PNG images supported yet)' )# % (filename, ex))
		if self.slide == True:	
			self.set_image (self.fetch_image())
			self.redraw_canvas()
		return True

	def on_drag_enter (self, drag_context, x, y, timestamp):
		self.redraw_canvas()
	
	def on_drag_leave (self, drag_context, timestamp):
		self.redraw_canvas()
	
	def on_drop (self, x, y, sel_data, timestamp):
		print "Data dropped ..."
		filename = ''
		# get text-elements in selection data
		txt = sel_data.get_text()
		if txt:
			if txt[-1] == '\n':
				txt = txt[:-1]
			txt.replace('\n', '\\n')
			# if it is a filename, use it
			if txt.startswith('file://'):
				filename = txt[7:]
				os.system("gconftool-2 -t string -s /desktop/gnome/background/picture_filename " + filename)
			else:
				screenlets.show_error(self, 'Invalid string: %s.' % txt)
		else:
			# else get uri-part of selection
			uris = sel_data.get_uris()
			if uris and len(uris)>0:
				#print "URIS: "+str(uris	)
				filename = uris[0][7:]
		if filename != '':
			#self.set_image(filename)
			self.image_filename = filename
	
	def on_draw (self, ctx):

		ctx.set_operator(cairo.OPERATOR_OVER)
		ctx.scale(self.scale, self.scale)
		if self.theme:
			# if something is dragged over, lighten up the whole thing
			if self.dragging_over:
				ctx.set_operator(cairo.OPERATOR_XOR)
			# render bg
			#self.theme['GnomeArt-bg.svg'].render_cairo(ctx)
			self.theme.render(ctx, 'GnomeArt-bg')
			# render image if set
			
			if self.__image != None:
				
				ctx.save()
				ctx.translate(self.image_offset_x, self.image_offset_y)
				ctx.scale(self.image_scale, self.image_scale)
				ctx.set_source_surface(self.__image, 0, 0)
				ctx.paint()
				ctx.restore()

			# render frame
			#self.theme['GnomeArt-frame.svg'].render_cairo(ctx)
			self.theme.render(ctx, 'GnomeArt-frame')
			# render glass
			#self.theme['GnomeArt-glass.svg'].render_cairo(ctx)

			home = commands.getoutput("echo $HOME")

			ctx.translate(60,170)
			if self.paint_menu == True : self.theme.render(ctx, 'menu')				
			ctx.translate(-60,-170)
			self.theme.render(ctx, 'GnomeArt-glass')
	def on_focus(self, event):
	
		self.paint_menu = True
		self.redraw_canvas()
		
	def on_unfocus(self, event):
		self.paint_menu = False
		self.redraw_canvas()
	def on_menuitem_select (self, id):
		"""handle MenuItem-events in right-click menu"""
		if id == "next wallpaper":
			# TODO: use DBus-call for this
			#self.switch_hide_show()
			
			self.set_image (self.fetch_image())
			self.redraw_canvas()
		if id == "visit":
			# TODO: use DBus-call for this
			#self.switch_hide_show()

			os.system('firefox ' + self.url + " &")
		if id == "start":
			self.slide = True
			self.update()
		if id == "stop":
			self.slide = False
		if id[:7] == "Install":
			# TODO: use DBus-call for this
			self.show_install_dialog()
			self.update()
		if id[:4] == "save":
			if self.engine == 'gnome_art':
				if self.folders != '': 
					os.system('cp ' + self.image_filename + ' ' + self.folders + '/' + self.imagenumber + '.jpg')
					screenlets.show_message(self,'The image has been saved')
				else:
					screenlets.show_message(self,'No folder to save')
			else:
				screenlets.show_message(self,'You can only save downloaded pictures...')
	def show_install_dialog (self):
		"""Craete/Show the install-dialog."""
		# create filter
		flt = gtk.FileFilter()
		flt.add_pattern('*.')

		# create dialog
		dlg = gtk.FileChooserDialog(action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,buttons=(gtk.STOCK_CANCEL, 
			gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
		dlg.set_current_folder(os.environ['HOME'])
		dlg.set_title(('Select a folder'))
		dlg.set_filter(flt)
		# run
		resp		= dlg.run()
		filename	= dlg.get_filename()
		dlg.destroy()
		if resp == gtk.RESPONSE_OK:
			# create new installer
			
			# try installing and show result dialog
			self.window.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
			self.folders = filename 

			self.window.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.LEFT_PTR))	
	def on_draw_shape (self, ctx):
		self.on_draw(ctx)
	def on_mouse_down(self,event):
			x, y = self.window.get_pointer()
			x /= (self.scale)
			y /= (self.scale)
			if y >= 170 and y <=195:
				if x >= 60 and x <= 86 :
					self.slide = False
					self.update()
				elif x >= 87 and x <= 109 :
					self.slide = True
					self.update()
				elif x >= 110 and x <= 136 :
					self.set_image (self.fetch_image())
					self.redraw_canvas()

	
# If the program is run directly or passed as an argument to the python
# interpreter then launch as new application
if __name__ == "__main__":
	# create session object here, the rest is done automagically
	import screenlets.session
	screenlets.session.create_session(GnomeArtScreenlet)

